/**
Main web application class.
**/
class GeneralLexer {
	public function new() {}

	private function getMinRegexpOccurance(regexp:EReg, s:String, minSoFar:Int) {
		if (regexp.match(s)) {
			return Std.Int(Math.min(regexp.matchedPos().pos, minSoFar));
		} else {
			return minSoFar;
		}
	}

	private function min(params: Array<Float>):Float {
		return Lambda.fold(params.iterator(), Math.min, params[0]);
	}

	public function next(): String {
		for (line in lines) {
			var s = line;
			var quotesStartMin = Lambda.fold(
				quotes.iterator(),
				function(regexp, min) {return getMinRegexpOccurance(regexp.begin, s, min);},
				s.length);
			var quotesEndMin = Lambda.fold(
				quotes.iterator(),
				function(regexp, min) {return getMinRegexpOccurance(regexp.end, s, min);},
				s.length);
			var whiteSpaceMin = Lambda.fold(
				whiteSpaces.iterator(),
				function(regexp, min) {return getMinRegexpOccurance(regexp, s, min);},
				s.length);
			var punctuationMin = Lambda.fold(
				punctuation.iterator(),
				function(regexp, min) {return getMinRegexpOccurance(regexp, s, min);},
				s.length);
			var lineJoinerMin = getMinRegexpOccurance(lineJoiner, s, s.length);

			var min = Std.Int(min([lineJoinerMin, whiteSpaceMin, quotesEndMin, quotesStartMin, punctuationMin]));

			if (min != 0) {
				var result = s.substr(0, min);
				s = s.substr(min);
				return result;
			} else {
				switch(0) {
					case quotesStartMin://everything before that, if any, is next symbol. If not then this is next symbol.
						//get to know what ending quote should have and search for it
					case quotesEndMin://everything before that is string
						//shouldn't be possible, just pass
					case whiteSpaceMin://everything before that, if any, is next symbol
						//shouldn't be possible, just pass
					case lineJoinerMin://everything before that is next symbol which might be continued on next line though
						//shouldn't be possible, just pass
					case punctuationMin://everything before that, if any, is next symbol. If not then this is next symbol.
						//return itself
					default: trace("impossible thing happened");
				}
			}
		}
	}

	private var lines:Array<String>;
	private var quotes:Array<{begin:EReg, end:EReg}>;
	private var lineJoiner:EReg;
	private var whiteSpaces:Array<EReg>;
	private var punctuation:Array<EReg>;
}


class A {
	public function new() {}

	public function fun() {
		trace("1");

		Reflect.setField(this, "fun", function() { trace("2"); });
	}
}

class Test {
	public static function main(){
		var a1: A = new A();
		var a2: A = new A();

		a1.fun(); a1.fun();
		a2.fun(); a2.fun();
	}
}

